unit testmwunit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  mwHighlighter, mwPasSyn, StdCtrls, mwCustomEdit, ExtCtrls, SpeedBar,
  Placemnt, Menus, StdActns, ActnList;

type
  TEditform = class(TForm)
    m1: TmwPasSyn;
    mwCustomEdit1: TmwCustomEdit;
    OpenDialog1: TOpenDialog;
    FormStorage1: TFormStorage;
    SaveDialog1: TSaveDialog;
    PopupMenu2: TPopupMenu;
    ActionList1: TActionList;
    EditCopy1: TEditCopy;
    EditCut1: TEditCut;
    EditPaste1: TEditPaste;
    Copy1: TMenuItem;
    Cut1: TMenuItem;
    Paste1: TMenuItem;
    EditDel1: TAction;
    Delete1: TMenuItem;
    EditUndo: TAction;
    EditRedo: TAction;
    Undo1: TMenuItem;
    Redo1: TMenuItem;
    N1: TMenuItem;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Exit1: TMenuItem;
    N3: TMenuItem;
    SaveAs1: TMenuItem;
    Save1: TMenuItem;
    Open1: TMenuItem;
    New1: TMenuItem;
    Edit1: TMenuItem;
    Undo2: TMenuItem;
    Redo2: TMenuItem;
    N2: TMenuItem;
    Cut2: TMenuItem;
    Copy2: TMenuItem;
    Paste2: TMenuItem;
    Delete2: TMenuItem;
    FileOpen: TAction;
    FileSave: TAction;
    FileSaveAs: TAction;
    FileExit: TAction;
    FindSearch: TAction;
    FindReplace: TAction;
    Find1: TMenuItem;
    Search1: TMenuItem;
    Replace1: TMenuItem;
    FileNew: TAction;
    procedure mwCustomEdit1Change(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure mwCustomEdit1ReplaceText(Sender: TObject; const ASearch,
      AReplace: String; Line, Column: Integer;
      var Action: TmwReplaceAction);
    procedure mwCustomEdit1DropFiles(Sender: TObject; X, Y: Integer;
      Files: TStrings);
    procedure loadfile(const filename : shortstring);
    procedure EditDel1Execute(Sender: TObject);
    function confirmChanges: boolean;
    procedure EditUndoExecute(Sender: TObject);
    procedure EditRedoExecute(Sender: TObject);
    procedure FileOpenExecute(Sender: TObject);
    procedure FileSaveExecute(Sender: TObject);
    procedure FileSaveAsExecute(Sender: TObject);
    procedure FileExitExecute(Sender: TObject);
    procedure FindSearchExecute(Sender: TObject);
    procedure FindReplaceExecute(Sender: TObject);
    procedure FileNewExecute(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Editform        : TEditform;
  named        : boolean = false;        // true if editing an existing file
  haschanged   : boolean = false;        // true if changed after last save
  currfileext  : shortstring ='';        // open file extention
  currFilename : string;                 // open file name (and path)
implementation

uses ReplaceDialog;

{$R *.DFM}

(************************************************************************************)
procedure TEditform.loadfile(const filename : shortstring);
begin
  // extract the extention to determine file type
  CurrFileExt := ANSIuppercase(extractfileext(filename));
  CurrfileName := filename;
  if (CurrfileExt <> '.PAS') AND (CurrFileExt <> '.DPR')
  // if it's a pascal file select the appropriate highlighter
     AND (CurrFileExt <> '.DFM') AND (CurrFileExt <> '.INC') then
     mwcustomedit1.HighLighter := nil
  else // otherwise just default to Windows colors
     mwcustomedit1.HighLighter := m1;
  try  // attempt to load the file and diasplay an error message if unsucsessful
    mwcustomedit1.Lines.LoadFromFile(filename);
  except
    application.messagebox('Cannot find the requested file','Error',mb_iconstop);
  end;
  named := true;
  haschanged := false;
end;
(************************************************************************************)
procedure TEditform.mwCustomEdit1Change(Sender: TObject);
begin
   haschanged := true;
end;
(************************************************************************************)
procedure TEditform.FormClose(Sender: TObject; var Action: TCloseAction);
begin
   if not confirmchanges then // if user didn't confirm it, then do not exit
      Action := canone;
end;
(************************************************************************************)
procedure TEditform.FormCreate(Sender: TObject);
begin
   // synchronize filters for open and save dialogs
   savedialog1.Filter := opendialog1.Filter;
   savedialog1.Filterindex := opendialog1.Filterindex;
   if fileexists(paramstr(1)) then // load any file given as a parameter
      loadfile(paramstr(1));
end;
(************************************************************************************)
procedure TEditform.mwCustomEdit1ReplaceText(Sender: TObject; const ASearch,
  AReplace: String; Line, Column: Integer; var Action: TmwReplaceAction);
begin                                       // handles replace text commands
  if dlgReplace.cbPrompt.Checked then       // checks for user confirmation if necessary
     case MessageDlg('Replace selected text ?', mtConfirmation,
                     [mbYes, mbNo, mbCancel, mbYesToAll], 0) of
        mrYes     : Action := mwraReplace;
        mrCancel  : Action := mwraCancel;
        mrYesToAll: Action := mwraReplaceAll;
        mrNo      : Action := mwraSkip;
     end;
end;
(************************************************************************************)
function TEditform.confirmchanges : boolean; // asks user if he/she wishes to save changes
var userreply : longint;     // before exiting/ opening another file/ creating a new file
begin
  result := false;
  if haschanged then         // only ask if the open file has been changed
     begin
       userreply := application.messagebox('The currently open file has been changed.'
                               +#10+#13+'Would you like to save these changes?',
                               'Save Changes...',MB_YESNOCANCEL);
       case userreply of
         IDYES :  begin
                    fileSave.Execute;       // try to save the open file
                    if not haschanged then  // if user presses save on the save dialog
                       result := true;      // then return true (ie: continue)
                  end;
         IDNO  :  result := true;    // just continue, regardless of current file status
       else
         result := false;            // do not continu, return to current open file
       end;
     end
  else
      result := true;
end;
(************************************************************************************)
procedure TEditform.mwCustomEdit1DropFiles(Sender: TObject; X, Y: Integer;
  Files: TStrings);
begin
  if confirmchanges then             // load any file dropped in the window
     loadfile(files[0]);             // after confirmation
end;
(************************************************************************************)
procedure TEditform.EditDel1Execute(Sender: TObject);
begin                                // delete selected text
  if length(mwcustomedit1.SelText) > 0 then
     mwcustomedit1.seltext := '';
end;
(************************************************************************************)
procedure TEditform.EditUndoExecute(Sender: TObject);
begin                                // undo last action
  mwcustomedit1.Undo;
end;
(************************************************************************************)
procedure TEditform.EditRedoExecute(Sender: TObject);
begin                                // redo last action
  mwcustomedit1.redo;
end;
(************************************************************************************)
procedure TEditform.FileOpenExecute(Sender: TObject);
begin
     if confirmchanges then
        begin
          m1.commentAttri.Style := [];   // reset the highlighter attributes
          m1.KeyAttri.style := [];
          m1.SymbolAttri.style := [];
          if opendialog1.execute then    // and try to open another file
             begin
               loadfile(opendialog1.filename);
               named := true;
             end;
          haschanged := false;
        end;
end;
(************************************************************************************)
procedure TEditform.FileSaveExecute(Sender: TObject);
begin                                    // saves the current file
   if named then
      begin
         mwcustomedit1.Lines.SaveToFile(opendialog1.filename);
         haschanged := false;
      end
   else
      filesaveas.Execute;                // if it's a new file then get a file name first
end;
(************************************************************************************)
procedure TEditform.FileSaveAsExecute(Sender: TObject);
begin                                    // save the current file under a different name
   if named then savedialog1.InitialDir := extractfilepath(CurrFileName);
   if savedialog1.Execute then
      begin
        try
          mwcustomedit1.Lines.SaveToFile(savedialog1.filename);
          CurrFileExt := ANSIuppercase(extractfileext(savedialog1.filename));
          currFileName := savedialog1.FileName;
          // select highlighter according to saved file extention
          if (currFileExt <> '.PAS') AND (CurrFileExt <> '.DPR')
             AND (CurrFileExt <> '.DFM') AND (CurrFileExt <> '.INC') then
             mwcustomedit1.HighLighter := nil
          else
             mwcustomedit1.HighLighter := m1;
          haschanged := false;
          named := true;
        except
          application.messagebox('Cannot write to the selected file','Error',mb_iconstop);
        end;
      end;
end;
(************************************************************************************)
procedure TEditform.FileExitExecute(Sender: TObject);
begin
    close;
end;
(************************************************************************************)
procedure TEditform.FindSearchExecute(Sender: TObject);
var
  OldBlockBegin: TPoint;
  OldBlockEnd: TPoint;
begin
   with mwcustomedit1 do                    // if anything is selected then place it in
     if Length(SelText) > 1 then            // the search edit control
        Dlgreplace.cbFindText.Text := SelText
     else
       begin                                // otherwise select the word the cursor is on
         OldBlockBegin := BlockBegin;       // and place that one in the search edit control
         OldBlockEnd := BlockEnd;
         SetSelWord;
         DlgReplace.cbFindText.Text := SelText;
         BlockBegin := OldBlockBegin;
         BlockEnd := OldBlockEnd;
       end;
   dlgreplace.btnreplace.Enabled := false;  // displabe the replace option controls 
   dlgreplace.btnreplaceall.enabled := false;
   dlgreplace.cbreplacetext.Enabled := false;
   dlgreplace.btnsearch.Enabled := true;
   dlgreplace.show;                          // display the search/replace dialog box
end;
(************************************************************************************)
procedure TEditform.FindReplaceExecute(Sender: TObject);
var
  OldBlockBegin: TPoint;
  OldBlockEnd: TPoint;
begin
   with mwcustomedit1 do                     // same as search above
     if Length(SelText) > 1 then
        Dlgreplace.cbFindText.Text := SelText
     else
       begin
         OldBlockBegin := BlockBegin;
         OldBlockEnd := BlockEnd;
         SetSelWord;
         DlgReplace.cbFindText.Text := SelText;
         BlockBegin := OldBlockBegin;
         BlockEnd := OldBlockEnd;
       end;
   dlgreplace.btnreplace.Enabled := true;  // this time enable the replace option controls
   dlgreplace.btnreplaceall.enabled := true;
   dlgreplace.cbreplacetext.Enabled := true;
   dlgreplace.btnsearch.Enabled := false;
   dlgreplace.show;
end;
(************************************************************************************)
procedure TEditform.FileNewExecute(Sender: TObject);
begin
  if confirmchanges then  // after confirmation throw away any currently open files
     begin                // and start a new one (unnamed)
       mwcustomedit1.ClearAll;
       named := false;
       haschanged := false;
     end;
end;
(************************************************************************************)
end.
